@@ -1,11 +1,10 @@
-#This file is part of Tryton.  The COPYRIGHT file at the top level of
-#this repository contains the full copyright notices and license terms.
+# This file is part of Tryton.  The COPYRIGHT file at the top level of
+# this repository contains the full copyright notices and license terms.
 "Options"
 import ConfigParser
 import optparse
 import os
 import gettext
-from version import VERSION
 import logging
 import sys
 import locale
@@ -13,6 +12,7 @@ import gtk
 import site
 
 from tryton.exceptions import TrytonError
+from tryton import __version__
 
 _ = gettext.gettext
 
@@ -26,9 +26,9 @@ def get_home_dir():
 def get_config_dir():
     if os.name == 'nt':
         return os.path.join(os.environ['APPDATA'], '.config', 'tryton',
-                VERSION.rsplit('.', 1)[0])
+                __version__.rsplit('.', 1)[0])
     return os.path.join(os.environ['HOME'], '.config', 'tryton',
-            VERSION.rsplit('.', 1)[0])
+            __version__.rsplit('.', 1)[0])
 if not os.path.isdir(get_config_dir()):
     os.makedirs(get_config_dir(), 0700)
 
@@ -37,10 +37,9 @@ class ConfigManager(object):
     "Config manager"
 
     def __init__(self):
-        short_version = '.'.join(VERSION.split('.', 2)[:2])
+        short_version = '.'.join(__version__.split('.', 2)[:2])
         demo_server = 'demo%s.tryton.org' % short_version
         demo_database = 'demo%s' % short_version
-        form_tab = 'left' if os.name != 'nt' else 'top'
         self.defaults = {
             'login.profile': demo_server,
             'login.login': 'demo',
@@ -55,7 +54,6 @@ class ConfigManager(object):
             'client.default_height': 750,
             'client.modepda': False,
             'client.toolbar': 'default',
-            'client.form_tab': form_tab,
             'client.maximize': False,
             'client.save_width_height': True,
             'client.save_tree_state': True,
@@ -78,7 +76,7 @@ class ConfigManager(object):
         self.arguments = []
 
     def parse(self):
-        parser = optparse.OptionParser(version=("Tryton %s" % VERSION),
+        parser = optparse.OptionParser(version=("Tryton %s" % __version__),
                 usage="Usage: %prog [options] [url]")
         parser.add_option("-c", "--config", dest="config",
                 help=_("specify alternate config file"))
@@ -181,7 +179,7 @@ else:
         unicode(os.path.dirname(__file__), sys.getfilesystemencoding()),
         '..')))
 
-for dir in [CURRENT_DIR, site.USER_BASE, sys.prefix]:
+for dir in [CURRENT_DIR, getattr(site, 'USER_BASE', sys.prefix), sys.prefix]:
     PIXMAPS_DIR = os.path.join(dir, 'share', 'pixmaps', 'tryton')
     if os.path.isdir(PIXMAPS_DIR):
         break
