@@ -1,19 +1,21 @@
 # -*- coding: utf-8 -*-
-#This file is part of Tryton.  The COPYRIGHT file at the top level of
-#this repository contains the full copyright notices and license terms.
+# This file is part of Tryton.  The COPYRIGHT file at the top level of
+# this repository contains the full copyright notices and license terms.
 import gtk
 import gettext
-import os
-from tryton.config import TRYTON_ICON, PIXMAPS_DIR, DATA_DIR
-from tryton.version import VERSION, WEBSITE
 import webbrowser
+from tryton.config import TRYTON_ICON
+from tryton.common import get_toplevel_window
+from tryton import __version__
 
 COPYRIGHT = '''\
-Copyright (C) 2007-2010 Cédric Krier.
-Copyright (C) 2007-2010 Bertrand Chenal.
-Copyright (C) 2008-2010 B2CK SPRL.
-Copyright (C) 2008-2010 Udo Spallek.
-Copyright (C) 2008-2010 virtual things - Preisler & Spallek GbR.
+Copyright (C) 2010-2011 Nicolas Évrard.
+Copyright (C) 2007-2011 Cédric Krier.
+Copyright (C) 2007-2011 Bertrand Chenal.
+Copyright (C) 2008-2011 B2CK SPRL.
+Copyright (C) 2008-2011 Udo Spallek.
+Copyright (C) 2008-2011 virtual things - Preisler & Spallek GbR.
+Copyright (C) 2007-2009 Lorenzo Gil Sanchez.
 Copyright (C) 2004-2008 Tiny SPRL.
 '''
 AUTHORS = [
@@ -22,7 +24,8 @@ AUTHORS = [
         'Franz Wiesinger',
         'Hartmut Goebel',
         'Korbinian Preisler <info@virtual-things.biz>',
-        'Mathias Behrle',
+        'Mathias Behrle <info@m9s.biz>',
+        'Nicolas Évrard <nicolas.evrard@b2ck.com>',
         'Sednacom <contact@sednacom.fr>',
         'Udo Spallek <info@virtual-things.biz>',
         ]
@@ -707,18 +710,19 @@ _ = gettext.gettext
 
 class About(object):
 
-    def __init__(self, parent):
+    def __init__(self):
         gtk.about_dialog_set_email_hook(lambda widget, link:
                 webbrowser.open(link, new=2))
         gtk.about_dialog_set_url_hook(lambda widget, link:
                 webbrowser.open(link, new=2))
+        parent = get_toplevel_window()
         self.win = gtk.AboutDialog()
         self.win.set_transient_for(parent)
         self.win.set_name('Tryton')
-        self.win.set_version(VERSION)
+        self.win.set_version(__version__)
         self.win.set_copyright(COPYRIGHT)
         self.win.set_license(LICENSE)
-        self.win.set_website(WEBSITE)
+        self.win.set_website('http://www.tryton.org/')
         self.win.set_authors(AUTHORS)
         self.win.set_logo(TRYTON_ICON)
 
