@@ -1,20 +1,20 @@
-#This file is part of Tryton.  The COPYRIGHT file at the top level of
-#this repository contains the full copyright notices and license terms.
+# This file is part of Tryton.  The COPYRIGHT file at the top level of
+# this repository contains the full copyright notices and license terms.
 import os
 from itertools import chain
 import tempfile
 import locale
-from tryton.common import datetime_strftime, \
+from tryton.common import \
         domain_inversion, eval_domain, localize_domain, \
-        merge, inverse_leaf, concat, simplify, EvalEnvironment
+        merge, inverse_leaf, filter_leaf, concat, simplify, unique_value, \
+        EvalEnvironment
 import tryton.common as common
-import time
 import datetime
 import decimal
 from decimal import Decimal
 import math
-from tryton.translate import date_format
 from tryton.common import RPCExecute, RPCException
+import tryton.rpc as rpc
 
 
 class Field(object):
@@ -88,14 +88,12 @@ class Field(object):
         elif domain == [('id', '=', None)]:
             res = False
         else:
-            if (isinstance(domain, list)
-                    and len(domain) == 1
-                    and domain[0][1] == '='):
+            unique, leftpart, value = unique_value(domain)
+            if unique:
                 # If the inverted domain is so constraint that only one value
                 # is possible we should use it. But we must also pay attention
                 # to the fact that the original domain might be a 'OR' domain
                 # and thus not preventing the modification of fields.
-                leftpart, _, value = domain[0][:3]
                 if value is False:
                     # XXX to remove once server domains are fixed
                     value = None
@@ -113,7 +111,7 @@ class Field(object):
                             constraintfields.add(leaf[0])
                     if localpart != 'id' or recordpart not in constraintfields:
                         setdefault = False
-                if setdefault:
+                if setdefault and not pre_validate:
                     self.set_client(record, value)
                     self.get_state_attrs(record)['domain_readonly'] = (
                         domain_readonly)
@@ -202,12 +200,18 @@ class DateTimeField(Field):
     _default = None
 
     def set_client(self, record, value, force_change=False):
-        if isinstance(value, basestring):
-            try:
-                value = datetime.datetime(*time.strptime(value,
-                        date_format() + ' ' + self.time_format(record))[:6])
-            except ValueError:
-                value = self._default
+        if isinstance(value, datetime.time):
+            current_value = self.get_client(record)
+            if current_value:
+                value = datetime.datetime.combine(
+                    current_value.date(), value)
+            else:
+                value = None
+        elif value and not isinstance(value, datetime.datetime):
+            current_value = self.get_client(record)
+            if current_value:
+                value = datetime.datetime.combine(
+                    value, current_value.time())
         if value:
             value = common.untimezoned_date(value)
         super(DateTimeField, self).set_client(record, value,
@@ -216,10 +220,11 @@ class DateTimeField(Field):
     def get_client(self, record):
         value = super(DateTimeField, self).get_client(record)
         if value:
-            value = common.timezoned_date(value)
-            return datetime_strftime(value, date_format() + ' ' +
-                self.time_format(record))
-        return ''
+            return common.timezoned_date(value)
+
+    def date_format(self, record):
+        context = self.context_get(record)
+        return context.get('date_format', '%x')
 
     def time_format(self, record):
         return record.expr_eval(self.attrs['format'])
@@ -230,23 +235,15 @@ class DateField(Field):
     _default = None
 
     def set_client(self, record, value, force_change=False):
-        if isinstance(value, basestring):
-            try:
-                value = datetime.date(*time.strptime(value,
-                        date_format())[:3])
-            except ValueError:
-                value = self._default
-        elif isinstance(value, datetime.datetime):
+        if isinstance(value, datetime.datetime):
             assert(value.time() == datetime.time())
             value = value.date()
         super(DateField, self).set_client(record, value,
             force_change=force_change)
 
-    def get_client(self, record):
-        value = super(DateField, self).get_client(record)
-        if value:
-            return datetime_strftime(value, date_format())
-        return ''
+    def date_format(self, record):
+        context = self.context_get(record)
+        return context.get('date_format', '%x')
 
 
 class TimeField(Field):
@@ -254,27 +251,34 @@ class TimeField(Field):
     _default = None
 
     def set_client(self, record, value, force_change=False):
-        if isinstance(value, basestring):
-            try:
-                value = datetime.time(*time.strptime(value,
-                        self.time_format(record))[3:6])
-            except ValueError:
-                value = self._default
-        elif isinstance(value, datetime.datetime):
+        if isinstance(value, datetime.datetime):
             value = value.time()
         super(TimeField, self).set_client(record, value,
             force_change=force_change)
 
-    def get_client(self, record):
-        value = super(TimeField, self).get_client(record)
-        if value is not None:
-            return value.strftime(self.time_format(record))
-        return ''
-
     def time_format(self, record):
         return record.expr_eval(self.attrs['format'])
 
 
+class TimeDeltaField(Field):
+
+    _default = None
+
+    def converter(self, record):
+        # TODO allow local context converter
+        return rpc.CONTEXT.get(self.attrs.get('converter'))
+
+    def set_client(self, record, value, force_change=False):
+        if isinstance(value, basestring):
+            value = common.timedelta.parse(value, self.converter(record))
+        super(TimeDeltaField, self).set_client(
+            record, value, force_change=force_change)
+
+    def get_client(self, record):
+        value = super(TimeDeltaField, self).get_client(record)
+        return common.timedelta.format(value, self.converter(record))
+
+
 class FloatField(Field):
     _default = None
     default_digits = (16, 2)
@@ -626,6 +630,9 @@ class O2MField(Field):
         self._connect_value(group)
 
     def set_client(self, record, value, force_change=False):
+        # domain inversion could try to set None as value
+        if value is None:
+            value = []
         # domain inversion could try to set id as value
         if isinstance(value, (int, long)):
             value = [value]
@@ -718,7 +725,7 @@ class O2MField(Field):
             else:
                 ldomain = [('id', '=', None)]
         for record2 in record.value.get(self.name, []):
-            if not record2.loaded and record2.id >= 0:
+            if not record2.loaded and record2.id >= 0 and not pre_validate:
                 continue
             test &= record2.validate(softvalidation=softvalidation,
                 pre_validate=ldomain)
@@ -826,17 +833,36 @@ class ReferenceField(Field):
                 skip={record.group.child_name})
         return super(ReferenceField, self).get_on_change_value(record)
 
+    def validation_domains(self, record, pre_validate=None):
+        screen_domain, attr_domain = self.domains_get(record, pre_validate)
+        return screen_domain
+
+    def domain_get(self, record):
+        if record.value.get(self.name):
+            model = record.value[self.name][0]
+        else:
+            model = None
+        screen_domain, attr_domain = self.domains_get(record)
+        return concat(localize_domain(
+                filter_leaf(screen_domain, self.name, model)), attr_domain)
+
+
+class _FileCache(object):
+    def __init__(self, path):
+        self.path = path
+
 
 class BinaryField(Field):
 
     _default = None
+    cast = bytearray if bytes == str else bytes
 
     def get(self, record):
         result = record.value.get(self.name, self._default)
-        if isinstance(result, basestring):
+        if isinstance(result, _FileCache):
             try:
-                with open(result, 'rb') as fp:
-                    result = buffer(fp.read())
+                with open(result.path, 'rb') as fp:
+                    result = self.cast(fp.read())
             except IOError:
                 result = self.get_data(record)
         return result
@@ -848,7 +874,7 @@ class BinaryField(Field):
         _, filename = tempfile.mkstemp(prefix='tryton_')
         with open(filename, 'wb') as fp:
             fp.write(value or '')
-        self.set(record, filename)
+        self.set(record, _FileCache(filename))
         record.modified_fields.setdefault(self.name)
         record.signal('record-modified')
         self.sig_changed(record)
@@ -857,14 +883,15 @@ class BinaryField(Field):
 
     def get_size(self, record):
         result = record.value.get(self.name) or 0
-        if isinstance(result, basestring):
-            result = os.stat(result).st_size
-        elif isinstance(result, buffer):
+        if isinstance(result, _FileCache):
+            result = os.stat(result.path).st_size
+        elif isinstance(result, (basestring, bytes, bytearray)):
             result = len(result)
         return result
 
     def get_data(self, record):
-        if not isinstance(record.value.get(self.name), (basestring, buffer)):
+        if not isinstance(record.value.get(self.name),
+                (basestring, bytes, bytearray)):
             if record.id < 0:
                 return ''
             context = record.context_get()
@@ -876,7 +903,7 @@ class BinaryField(Field):
             _, filename = tempfile.mkstemp(prefix='tryton_')
             with open(filename, 'wb') as fp:
                 fp.write(values[self.name] or '')
-            self.set(record, filename)
+            self.set(record, _FileCache(filename))
         return self.get(record)
 
 
@@ -899,9 +926,15 @@ class DictField(Field):
         return concat(localize_domain(inverse_leaf(screen_domain)),
             attr_domain)
 
+    def date_format(self, record):
+        context = self.context_get(record)
+        return context.get('date_format', '%x')
+
+    def time_format(self, record):
+        return '%X'
+
 TYPES = {
     'char': CharField,
-    'float_time': FloatField,
     'integer': IntegerField,
     'biginteger': IntegerField,
     'float': FloatField,
@@ -915,6 +948,7 @@ TYPES = {
     'datetime': DateTimeField,
     'date': DateField,
     'time': TimeField,
+    'timedelta': TimeDeltaField,
     'one2one': O2OField,
     'binary': BinaryField,
     'dict': DictField,
