@@ -1,5 +1,5 @@
-#This file is part of Tryton.  The COPYRIGHT file at the top level of
-#this repository contains the full copyright notices and license terms.
+# This file is part of Tryton.  The COPYRIGHT file at the top level of
+# this repository contains the full copyright notices and license terms.
 import tryton.rpc as rpc
 from tryton.signal_event import SignalEvent
 import tryton.common as common
@@ -84,7 +84,7 @@ class Record(SignalEvent):
                             # Don't compute context for same group
                             or (record.context_get() == record_context)))
 
-                if self.parent:
+                if self.parent and self.parent.model_name == self.model_name:
                     group = sum(self.parent.group.children, [])
                     filter_ = filter_parent_group
                 else:
@@ -270,6 +270,7 @@ class Record(SignalEvent):
     def cancel(self):
         self._loaded.clear()
         self.modified_fields.clear()
+        self._timestamp = None
 
     def get_timestamp(self):
         result = {self.model_name + ',' + str(self.id): self._timestamp}
@@ -311,8 +312,7 @@ class Record(SignalEvent):
                             [self.id], value, context=context)
                     except RPCException:
                         return False
-            self._loaded.clear()
-            self.modified_fields = {}
+            self.cancel()
             if force_reload:
                 self.reload()
             if self.group:
