@@ -1,5 +1,5 @@
-#This file is part of Tryton.  The COPYRIGHT file at the top level of
-#this repository contains the full copyright notices and license terms.
+# This file is part of Tryton.  The COPYRIGHT file at the top level of
+# this repository contains the full copyright notices and license terms.
 "Screen"
 import copy
 import gobject
@@ -886,6 +886,8 @@ class Screen(SignalEvent):
         def is_active(record, button):
             if record.group.readonly or record.readonly:
                 return False
+            if button.attrs.get('type', 'class') == 'instance':
+                return False
             states = record.expr_eval(button.attrs.get('states', {}))
             return not (states.get('invisible') or states.get('readonly'))
 
@@ -902,11 +904,7 @@ class Screen(SignalEvent):
 
     def button(self, button):
         'Execute button on the selected records'
-        if button.get('confirm', False) and not sur(button['confirm']):
-            return
         self.current_view.set_value()
-        if not self.current_record.save(force_reload=False):
-            return
         fields = self.current_view.get_fields()
         for record in self.selected_records:
             domain = record.expr_eval(
@@ -917,10 +915,37 @@ class Screen(SignalEvent):
                     # Reset valid state with normal domain
                     record.validate(fields)
                 return
+        if button.get('confirm', False) and not sur(button['confirm']):
+            return
+        if button.get('type', 'class') == 'class':
+            if not self.current_record.save(force_reload=False):
+                return
+        if button.get('type', 'class') == 'class':
+            self._button_class(button)
+        else:
+            self._button_instance(button)
+
+    def _button_instance(self, button):
+        record = self.current_record
+        args = record.expr_eval(button.get('change', []))
+        values = record._get_on_change_args(args)
+        try:
+            changes = RPCExecute('model', self.model_name, button['name'],
+                values, context=self.context)
+        except RPCException:
+            return
+        record.set_on_change(changes)
+        record.signal('record-changed')
+
+    def _button_class(self, button):
         ids = [r.id for r in self.selected_records]
+        context = self.context.copy()
+        context['_timestamp'] = {}
+        for record in self.selected_records:
+            context['_timestamp'].update(record.get_timestamp())
         try:
             action = RPCExecute('model', self.model_name, button['name'],
-                ids, context=self.context)
+                ids, context=context)
         except RPCException:
             action = None
         self.reload(ids, written=True)
@@ -929,7 +954,7 @@ class Screen(SignalEvent):
         elif action:
             Action.execute(action, {
                     'model': self.model_name,
-                    'id': record.id,
+                    'id': self.current_record.id,
                     'ids': ids,
                     }, context=self.context)
 
@@ -958,6 +983,10 @@ class Screen(SignalEvent):
         elif action.startswith('switch'):
             _, view_type = action.split(None, 1)
             self.switch_view(view_type=view_type)
+        elif action == 'reload':
+            if (self.current_view.view_type in ['tree', 'graph', 'calendar']
+                    and not self.parent):
+                self.search_filter()
         elif action == 'reload menu':
             from tryton.gui import Main
             RPCContextReload(Main.get_main().sig_win_menu)
