@@ -1,7 +1,8 @@
-#This file is part of Tryton.  The COPYRIGHT file at the top level of
-#this repository contains the full copyright notices and license terms.
+# This file is part of Tryton.  The COPYRIGHT file at the top level of
+# this repository contains the full copyright notices and license terms.
 import gtk
 import gettext
+import itertools
 
 from .widget import Widget
 from tryton.gui.window.view_form.screen import Screen
@@ -10,6 +11,7 @@ from tryton.gui.window.win_form import WinForm
 import tryton.common as common
 from tryton.common.placeholder_entry import PlaceholderEntry
 from tryton.common.completion import get_completion, update_completion
+from tryton.common.domain_parser import quote
 
 _ = gettext.gettext
 
@@ -48,7 +50,10 @@ class One2Many(Widget):
             hbox.pack_start(self.wid_text, expand=True, fill=True)
 
             if int(self.attrs.get('completion', 1)):
-                self.wid_completion = get_completion()
+                access = common.MODELACCESS[attrs['relation']]
+                self.wid_completion = get_completion(
+                    search=access['read'] and access['write'],
+                    create=attrs.get('create', True) and access['create'])
                 self.wid_completion.connect('match-selected',
                     self._completion_match_selected)
                 self.wid_completion.connect('action-activated',
@@ -287,7 +292,6 @@ class One2Many(Widget):
                 self._position
                 and self._position > 1))
         if self.attrs.get('add_remove'):
-            self.wid_text.set_sensitive(not self._readonly)
             self.but_add.set_sensitive(bool(
                     not self._readonly
                     and not size_limit
@@ -298,10 +302,11 @@ class One2Many(Widget):
                     and self._position
                     and access['write']
                     and access['read']))
+            self.wid_text.set_sensitive(self.but_add.get_sensitive())
 
         # New button must be added to focus chain to allow keyboard only
         # creation when there is no existing record on form view.
-        focus_chain = self.title_box.get_focus_chain()
+        focus_chain = self.title_box.get_focus_chain() or []
         if o2m_size == 0 and self.screen.current_view.view_type == 'form':
             if self.but_new not in focus_chain:
                 focus_chain.append(self.but_new)
@@ -322,11 +327,18 @@ class One2Many(Widget):
                 return False
         return True
 
-    def _sig_new(self, widget=None):
+    def _sig_new(self, *args):
         if not common.MODELACCESS[self.screen.model_name]['create']:
             return
         if not self._validate():
             return
+
+        if self.attrs.get('product'):
+            self._new_product()
+        else:
+            self._new_single()
+
+    def _new_single(self):
         ctx = {}
         ctx.update(self.field.context_get(self.record))
         sequence = None
@@ -350,6 +362,56 @@ class One2Many(Widget):
             WinForm(self.screen, lambda a: update_sequence(), new=True,
                 many=field_size, context=ctx)
 
+    def _new_product(self):
+        fields = self.attrs['product'].split(',')
+        product = {}
+
+        first = self.screen.new(default=False)
+        default = first.default_get()
+        first.set_default(default)
+
+        def search_set(*args):
+            if not fields:
+                return make_product()
+            field = self.screen.group.fields[fields.pop()]
+            relation = field.attrs.get('relation')
+            if not relation:
+                search_set()
+
+            domain = field.domain_get(first)
+            context = field.context_get(first)
+
+            def callback(result):
+                if result:
+                    product[field.name] = result
+
+            win_search = WinSearch(relation, callback, sel_multi=True,
+                context=context, domain=domain)
+            win_search.win.connect('destroy', search_set)
+            win_search.screen.search_filter()
+            win_search.show()
+
+        def make_product(first=first):
+            if not product:
+                self.screen.group.remove(first, remove=True)
+                return
+
+            fields = product.keys()
+            for values in itertools.product(*product.values()):
+                if first:
+                    record = first
+                    first = None
+                else:
+                    record = self.screen.new(default=False)
+                default_value = default.copy()
+                for field, value in zip(fields, values):
+                    id_, rec_name = value
+                    default_value[field] = id_
+                    default_value[field + '.rec_name'] = rec_name
+                record.set_default(default_value)
+
+        search_set()
+
     def _sig_edit(self, widget=None):
         if not common.MODELACCESS[self.screen.model_name]['read']:
             return
@@ -418,7 +480,8 @@ class One2Many(Widget):
             view_ids=self.attrs.get('view_ids', '').split(','),
             views_preload=self.attrs.get('views', {}),
             new=self.but_new.get_property('sensitive'))
-        win.screen.search_filter(text)
+        win.screen.search_filter(quote(text))
+        win.show()
 
     def _sig_label(self, screen, signal_data):
         self._position = signal_data[0]
