@@ -1,5 +1,5 @@
-#This file is part of Tryton.  The COPYRIGHT file at the top level of
-#this repository contains the full copyright notices and license terms.
+# This file is part of Tryton.  The COPYRIGHT file at the top level of
+# this repository contains the full copyright notices and license terms.
 import gtk
 import gettext
 
@@ -129,6 +129,9 @@ class Reference(Many2One, SelectionMixin, PopdownMixin):
                 active = i
                 break
         self.widget_combo.set_active(active)
+        if active == -1:
+            # When setting no item GTK doesn't clear the entry
+            self.widget_combo.child.set_text('')
 
     def display(self, record, field):
         self.update_selection(record, field)
