@@ -1,5 +1,5 @@
-#This file is part of Tryton.  The COPYRIGHT file at the top level of
-#this repository contains the full copyright notices and license terms.
+# This file is part of Tryton.  The COPYRIGHT file at the top level of
+# this repository contains the full copyright notices and license terms.
 import gtk
 import gobject
 
@@ -68,6 +68,8 @@ class Selection(Widget, SelectionMixin, PopdownMixin):
         self.set_popdown(self.selection, self.entry)
         if not field:
             self.entry.set_active(-1)
+            # When setting no item GTK doesn't clear the entry
+            self.entry.child.set_text('')
             return
         super(Selection, self).display(record, field)
         value = field.get(record)
@@ -77,6 +79,7 @@ class Selection(Widget, SelectionMixin, PopdownMixin):
 
         self.entry.handler_block_by_func(self.changed)
         if not self.set_popdown_value(self.entry, value):
-            self.get_inactive_selection(value)
+            text = self.get_inactive_selection(value)
+            self.set_popdown(self.selection[:] + [(value, text)], self.entry)
             self.set_popdown_value(self.entry, value)
         self.entry.handler_unblock_by_func(self.changed)
