@@ -1,7 +1,7 @@
-#This file is part of Tryton.  The COPYRIGHT file at the top level of
-#this repository contains the full copyright notices and license terms.
-#This code is inspired by the pycha project
-#(http://www.lorenzogil.com/projects/pycha/)
+# This file is part of Tryton.  The COPYRIGHT file at the top level of
+# this repository contains the full copyright notices and license terms.
+# This code is inspired by the pycha project
+# (http://www.lorenzogil.com/projects/pycha/)
 import gtk
 from functools import reduce
 from tryton.common import hex2rgb, generateColorscheme, \
@@ -15,7 +15,6 @@ import tryton.rpc as rpc
 import cairo
 from tryton.action import Action
 from tryton.gui.window import Window
-from tryton.translate import date_format
 
 
 class Popup(object):
@@ -376,12 +375,16 @@ class Graph(gtk.DrawingArea):
                 if yfield['name'] == '#':
                     self.datas[x][key] += 1
                 else:
-                    self.datas[x][key] += \
-                        float(model[yfield['name']].get(model) or 0)
+                    value = model[yfield['name']].get(model)
+                    if isinstance(value, datetime.timedelta):
+                        value = value.total_seconds()
+                    self.datas[x][key] += float(value or 0)
+        date_format = self.view.screen.context.get('date_format', '%x')
+        datetime_format = date_format + ' %X'
         if isinstance(minx, datetime.datetime):
             date = minx
             while date <= maxx:
-                self.labels[date] = datetime_strftime(date, date_format())
+                self.labels[date] = datetime_strftime(date, datetime_format)
                 self.datas.setdefault(date, {})
                 for yfield in self.yfields:
                     self.datas[date].setdefault(
@@ -390,7 +393,7 @@ class Graph(gtk.DrawingArea):
         elif isinstance(minx, datetime.date):
             date = minx
             while date <= maxx:
-                self.labels[date] = datetime_strftime(date, date_format())
+                self.labels[date] = datetime_strftime(date, date_format)
                 self.datas.setdefault(date, {})
                 for yfield in self.yfields:
                     self.datas[date].setdefault(
@@ -454,7 +457,7 @@ class Graph(gtk.DrawingArea):
         color = self.attrs.get('color', 'blue')
         r, g, b = hex2rgb(COLOR_SCHEMES.get(color, color))
         maxcolor = max(max(r, g), b)
-        self.colorScheme = generateColorscheme(color, keys + ['__highlight'],
+        self.colorScheme = generateColorscheme(color, keys,
                 maxcolor / (len(keys) or 1))
         for yfield in self.yfields:
             if yfield.get('color'):
