@@ -1,5 +1,5 @@
-#This file is part of Tryton.  The COPYRIGHT file at the top level of
-#this repository contains the full copyright notices and license terms.
+# This file is part of Tryton.  The COPYRIGHT file at the top level of
+# this repository contains the full copyright notices and license terms.
 
 import os
 import tempfile
@@ -13,11 +13,9 @@ from functools import wraps, partial
 from tryton.gui.window.win_search import WinSearch
 from tryton.gui.window.win_form import WinForm
 from tryton.gui.window.view_form.screen import Screen
-import tryton.rpc as rpc
 from tryton.common import COLORS, file_selection, file_open, slugify
 import tryton.common as common
 from tryton.common.cellrendererbutton import CellRendererButton
-from tryton.common.cellrendererdate import CellRendererDate
 from tryton.common.cellrenderertext import CellRendererText, \
     CellRendererTextCompletion
 from tryton.common.cellrenderertoggle import CellRendererToggle
@@ -27,10 +25,12 @@ from tryton.common.cellrendererfloat import CellRendererFloat
 from tryton.common.cellrendererbinary import CellRendererBinary
 from tryton.common.cellrendererclickablepixbuf import \
     CellRendererClickablePixbuf
-from tryton.translate import date_format
 from tryton.common import data2pixbuf
 from tryton.common.completion import get_completion, update_completion
 from tryton.common.selection import SelectionMixin, PopdownMixin
+from tryton.common.datetime_ import CellRendererDate, CellRendererTime
+from tryton.common.datetime_strftime import datetime_strftime
+from tryton.common.domain_parser import quote
 
 _ = gettext.gettext
 
@@ -108,7 +108,13 @@ class CellCache(list):
         return wrapper
 
 
-class Affix(object):
+class Cell(object):
+
+    def get_color(self, record):
+        return record.expr_eval(self.view.attributes.get('colors', '"black"'))
+
+
+class Affix(Cell):
 
     def __init__(self, view, attrs, protocol=None):
         super(Affix, self).__init__()
@@ -148,6 +154,12 @@ class Affix(object):
             if not text:
                 text = field.get_client(record) or ''
             cell.set_property('text', text)
+            fg_color = self.get_color(record)
+            cell.set_property('foreground', fg_color)
+            if fg_color == 'black':
+                cell.set_property('foreground-set', False)
+            else:
+                cell.set_property('foreground-set', True)
 
     def clicked(self, renderer, path):
         store = self.view.treeview.get_model()
@@ -163,10 +175,10 @@ class Affix(object):
             webbrowser.open(value, new=2)
 
 
-class Char(object):
+class GenericText(Cell):
 
     def __init__(self, view, attrs, renderer=None):
-        super(Char, self).__init__()
+        super(GenericText, self).__init__()
         self.attrs = attrs
         if renderer is None:
             renderer = CellRendererText
@@ -210,7 +222,7 @@ class Char(object):
 
         if self.attrs.get('type', field.attrs.get('type')) in \
                 ('float', 'integer', 'biginteger', 'boolean',
-                'numeric', 'float_time'):
+                'numeric', 'timedelta'):
             align = 1
         else:
             align = 0
@@ -255,9 +267,6 @@ class Char(object):
 
         cell.set_property('xalign', align)
 
-    def get_color(self, record):
-        return record.expr_eval(self.view.attributes.get('colors', '"black"'))
-
     def open_remote(self, record, create, changed=False, text=None,
             callback=None):
         raise NotImplementedError
@@ -283,7 +292,20 @@ class Char(object):
         return record, field
 
 
-class Int(Char):
+class Char(GenericText):
+
+    @realized
+    @CellCache.cache
+    def setter(self, column, cell, store, iter_):
+        super(Char, self).setter(column, cell, store, iter_)
+        cell.set_property('single-paragraph-mode', True)
+
+
+class Text(GenericText):
+    pass
+
+
+class Int(GenericText):
 
     def __init__(self, view, attrs, renderer=None):
         if renderer is None:
@@ -304,7 +326,7 @@ class Int(Char):
             callback()
 
 
-class Boolean(Char):
+class Boolean(GenericText):
 
     def __init__(self, view, attrs=None,
             renderer=None):
@@ -338,35 +360,61 @@ class URL(Char):
         cell.set_property('visible', not readonly)
 
 
-class Date(Char):
+class Date(GenericText):
 
     def __init__(self, view, attrs, renderer=None):
         if renderer is None:
-            renderer = partial(CellRendererDate, date_format())
+            renderer = CellRendererDate
         super(Date, self).__init__(view, attrs, renderer=renderer)
-        self.renderer.connect('editing-started', self.editing_started)
-
-
-class Datetime(Date):
 
     @realized
     def setter(self, column, cell, store, iter):
-        super(Datetime, self).setter(column, cell, store, iter)
         record = store.get_value(iter, 0)
         field = record[self.attrs['name']]
-        time_format = field.time_format(record)
-        self.renderer.format = date_format() + ' ' + time_format
+        self.renderer.props.format = self.get_format(record, field)
+        super(Date, self).setter(column, cell, store, iter)
+
+    def get_format(self, record, field):
+        if field and record:
+            return field.date_format(record)
+        else:
+            return '%x'
+
+    def get_textual_value(self, record):
+        if not record:
+            return ''
+        value = record[self.attrs['name']].get_client(record)
+        if value:
+            return datetime_strftime(value, self.renderer.props.format)
+        else:
+            return ''
 
 
 class Time(Date):
 
-    @realized
-    def setter(self, column, cell, store, iter):
-        super(Time, self).setter(column, cell, store, iter)
-        record = store.get_value(iter, 0)
-        field = record[self.attrs['name']]
-        time_format = field.time_format(record)
-        self.renderer.format = time_format
+    def __init__(self, view, attrs, renderer=None):
+        if renderer is None:
+            renderer = CellRendererTime
+        super(Time, self).__init__(view, attrs, renderer=renderer)
+
+    def get_format(self, record, field):
+        if field and record:
+            return field.time_format(record)
+        else:
+            return '%X'
+
+    def get_textual_value(self, record):
+        if not record:
+            return ''
+        value = record[self.attrs['name']].get_client(record)
+        if value:
+            return value.strftime(self.renderer.props.format)
+        else:
+            return ''
+
+
+class TimeDelta(GenericText):
+    pass
 
 
 class Float(Int):
@@ -385,28 +433,7 @@ class Float(Int):
         cell.digits = digits
 
 
-class FloatTime(Char):
-
-    def __init__(self, view, attrs, renderer=None):
-        super(FloatTime, self).__init__(view, attrs, renderer=renderer)
-        self.conv = None
-        if attrs and attrs.get('float_time'):
-            self.conv = rpc.CONTEXT.get(attrs['float_time'])
-
-    def get_textual_value(self, record):
-        val = record[self.attrs['name']].get(record)
-        return common.float_time_to_text(val, self.conv)
-
-    def value_from_text(self, record, text, callback=None):
-        field = record[self.attrs['name']]
-        digits = field.digits(record)
-        field.set_client(record,
-            common.text_to_float_time(text, self.conv, digits[1]))
-        if callback:
-            callback()
-
-
-class Binary(Char):
+class Binary(GenericText):
 
     def __init__(self, view, attrs, renderer=None):
         self.filename = attrs.get('filename')
@@ -509,7 +536,7 @@ class Binary(Char):
         field.set_client(record, False)
 
 
-class Image(Char):
+class Image(GenericText):
 
     def __init__(self, view, attrs=None, renderer=None):
         if renderer is None:
@@ -530,13 +557,14 @@ class Image(Char):
             else:
                 value = field.get_data(record)
         pixbuf = data2pixbuf(value)
-        if self.attrs['width'] != -1 or self.attrs['height'] != -1:
+        if (self.attrs.get('width', -1) != -1 or
+                self.attrs.get('height', -1) != -1):
             pixbuf = common.resize_pixbuf(pixbuf,
                 self.attrs['width'], self.attrs['height'])
         cell.set_property('pixbuf', pixbuf)
 
 
-class M2O(Char):
+class M2O(GenericText):
 
     def __init__(self, view, attrs, renderer=None):
         if renderer is None and int(attrs.get('completion', 1)):
@@ -554,8 +582,12 @@ class M2O(Char):
         relation = record[self.attrs['name']].attrs['relation']
         domain = record[self.attrs['name']].domain_get(record)
         context = record[self.attrs['name']].context_get(record)
-        self.search_remote(record, relation, text, domain=domain,
+        win = self.search_remote(record, relation, text, domain=domain,
             context=context, callback=callback)
+        if len(win.screen.group) == 1:
+            win.response(None, gtk.RESPONSE_OK)
+        else:
+            win.show()
 
     def open_remote(self, record, create=True, changed=False, text=None,
             callback=None):
@@ -563,7 +595,8 @@ class M2O(Char):
         relation = field.attrs['relation']
 
         access = common.MODELACCESS[relation]
-        if create and not access['create']:
+        if (create
+                and not (self.attrs.get('create', True) and access['create'])):
             return
         elif not access['read']:
             return
@@ -576,7 +609,7 @@ class M2O(Char):
             obj_id = field.get(record)
         else:
             self.search_remote(record, relation, text, domain=domain,
-                context=context, callback=callback)
+                context=context, callback=callback).show()
             return
         screen = Screen(relation, domain=domain, context=context,
             mode=['form'])
@@ -607,12 +640,16 @@ class M2O(Char):
                 callback()
         win = WinSearch(relation, search_callback, sel_multi=False,
             context=context, domain=domain)
-        win.screen.search_filter(text.decode('utf-8'))
+        win.screen.search_filter(quote(text.decode('utf-8')))
+        return win
 
     def set_completion(self, entry, path):
         if entry.get_completion():
             entry.set_completion(None)
-        completion = get_completion()
+        access = common.MODELACCESS[self.attrs['relation']]
+        completion = get_completion(
+            search=access['read'],
+            create=self.attrs.get('create', True) and access['create'])
         completion.connect('match-selected', self._completion_match_selected,
             path)
         completion.connect('action-activated',
@@ -665,7 +702,7 @@ class O2O(M2O):
     pass
 
 
-class O2M(Char):
+class O2M(GenericText):
 
     @realized
     def setter(self, column, cell, store, iter):
@@ -723,7 +760,7 @@ class M2M(O2M):
             context=context)
 
 
-class Selection(Char, SelectionMixin, PopdownMixin):
+class Selection(GenericText, SelectionMixin, PopdownMixin):
 
     def __init__(self, *args):
         super(Selection, self).__init__(*args)
@@ -775,7 +812,7 @@ class Selection(Char, SelectionMixin, PopdownMixin):
         return False
 
 
-class Reference(Char, SelectionMixin):
+class Reference(GenericText, SelectionMixin):
 
     def __init__(self, view, attrs, renderer=None):
         super(Reference, self).__init__(view, attrs, renderer=renderer)
@@ -809,6 +846,7 @@ class ProgressBar(object):
 
     def __init__(self, view, attrs):
         super(ProgressBar, self).__init__()
+        self.view = view
         self.attrs = attrs
         self.renderer = gtk.CellRendererProgress()
         orientation = self.orientations.get(self.attrs.get('orientation',
