@@ -1,5 +1,5 @@
-#This file is part of Tryton.  The COPYRIGHT file at the top level of
-#this repository contains the full copyright notices and license terms.
+# This file is part of Tryton.  The COPYRIGHT file at the top level of
+# this repository contains the full copyright notices and license terms.
 import gtk
 import gettext
 import gobject
@@ -8,7 +8,7 @@ import tryton.common as common
 from tryton.common.domain_parser import quote
 from tryton.common.placeholder_entry import PlaceholderEntry
 from tryton.common.treeviewcontrol import TreeViewControl
-from tryton.translate import date_format
+from tryton.common.datetime_ import Date, Time, DateTime, add_operators
 from tryton.config import TRYTON_ICON
 from tryton.pyson import PYSONDecoder
 
@@ -17,17 +17,25 @@ _ = gettext.gettext
 
 class Dates(gtk.HBox):
 
-    def __init__(self, format_):
+    def __init__(self, format_=None, _entry=Date):
         super(Dates, self).__init__()
-        self.from_ = common.date_widget.DateEntry(format_)
+        self.from_ = add_operators(_entry())
         self.pack_start(self.from_, expand=True, fill=True)
         self.pack_start(gtk.Label(_('..')), expand=False, fill=False)
-        self.to = common.date_widget.DateEntry(format_)
+        self.to = add_operators(_entry())
         self.pack_start(self.to, expand=True, fill=True)
+        if format_:
+            self.from_.props.format = format_
+            self.to.props.format = format_
+
+    def _get_value(self, widget):
+        value = widget.props.value
+        if value:
+            return common.datetime_strftime(value, widget.props.format)
 
     def get_value(self):
-        from_ = self.from_.get_text()
-        to = self.to.get_text()
+        from_ = self._get_value(self.from_)
+        to = self._get_value(self.to)
         if from_ and to:
             if from_ != to:
                 return '%s..%s' % (quote(from_), quote(to))
@@ -38,6 +46,47 @@ class Dates(gtk.HBox):
         elif to:
             return '<=%s' % quote(to)
 
+    def connect_activate(self, callback):
+        self.from_.connect('activate', callback)
+        self.to.connect('activate', callback)
+
+    def set_values(self, from_, to):
+        self.from_.props.value = from_
+        self.to.props.value = to
+
+
+class Times(Dates):
+
+    def __init__(self, format_, _entry=Time):
+        super(Times, self).__init__(_entry=_entry)
+
+    def connect_activate(self, callback):
+        for widget in self.from_.get_children() + self.to.get_children():
+            widget.child.connect('activate', callback)
+
+
+class DateTimes(Dates):
+
+    def __init__(self, date_format, time_format, _entry=DateTime):
+        super(DateTimes, self).__init__(_entry=_entry)
+        self.from_.props.date_format = date_format
+        self.to.props.date_format = date_format
+        self.from_.props.time_format = time_format
+        self.to.props.time_format = time_format
+
+    def _get_value(self, widget):
+        value = widget.props.value
+        if value:
+            return common.datetime_strftime(value,
+                widget.props.date_format + ' ' + widget.props.time_format)
+
+    def connect_activate(self, callback):
+        for widget in self.from_.get_children() + self.to.get_children():
+            if isinstance(widget, Date):
+                widget.connect('activate', callback)
+            elif isinstance(widget, Time):
+                widget.child.connect('activate', callback)
+
 
 class Selection(gtk.ScrolledWindow):
 
@@ -274,10 +323,6 @@ class ScreenContainer(object):
         for r in res:
             model.append([r.strip()])
 
-    def clear(self, widget=None):
-        self.search_entry.set_text('')
-        self.bookmark_match()
-
     def get_text(self):
         return self.search_entry.get_text().decode('utf-8')
 
@@ -420,12 +465,10 @@ class ScreenContainer(object):
                 if value is not None:
                     text += quote(label) + ': ' + value + ' '
             self.set_text(text)
-            self.last_search_text = self.get_text()
             self.do_search()
-
-        def date_activate(entry):
-            entry._focus_out(entry, None)
-            search()
+            # Store text after doing the search
+            # because domain parser could simplify the text
+            self.last_search_text = self.get_text()
 
         if not self.search_window:
             self.search_window = gtk.Window()
@@ -469,15 +512,16 @@ class ScreenContainer(object):
                     entry = Selection(selections)
                     yoptions = gtk.FILL | gtk.EXPAND
                 elif field['type'] in ('date', 'datetime', 'time'):
+                    date_format = self.screen.context.get('date_format', '%x')
                     if field['type'] == 'date':
-                        format_ = date_format()
+                        entry = Dates(date_format)
                     elif field['type'] in ('datetime', 'time'):
-                        format_ = PYSONDecoder({}).decode(field['format'])
-                        if field['type'] == 'datetime':
-                            format_ = date_format() + ' ' + format_
-                    entry = Dates(format_)
-                    entry.from_.connect('activate', date_activate)
-                    entry.to.connect('activate', date_activate)
+                        time_format = PYSONDecoder({}).decode(field['format'])
+                        if field['type'] == 'time':
+                            entry = Times(time_format)
+                        elif field['type'] == 'datetime':
+                            entry = DateTimes(date_format, time_format)
+                    entry.connect_activate(lambda *a: search())
                 else:
                     entry = gtk.Entry()
                     entry.connect('activate', lambda *a: search())
@@ -539,8 +583,7 @@ class ScreenContainer(object):
                     if isinstance(entry, gtk.ComboBox):
                         entry.set_active(-1)
                     elif isinstance(entry, Dates):
-                        entry.from_.set_text('')
-                        entry.to.set_text('')
+                        entry.set_values(None, None)
                     elif isinstance(entry, Selection):
                         entry.treeview.get_selection().unselect_all()
                     else:
