@@ -1,5 +1,5 @@
-#This file is part of Tryton.  The COPYRIGHT file at the top level of
-#this repository contains the full copyright notices and license terms.
+# This file is part of Tryton.  The COPYRIGHT file at the top level of
+# this repository contains the full copyright notices and license terms.
 import gtk
 import pango
 import gettext
@@ -107,6 +107,12 @@ class Wizard(object):
 
             def execute_actions():
                 for action in result.get('actions', []):
+                    for k, v in [
+                            ('direct_print', self.direct_print),
+                            ('email_print', self.email_print),
+                            ('email', self.email),
+                            ]:
+                        action[0].setdefault(k, v)
                     Action._exec_action(*action, context=self.context.copy())
 
             if self.state == self.end_state:
