@@ -1,13 +1,13 @@
-#This file is part of Tryton.  The COPYRIGHT file at the top level of
-#this repository contains the full copyright notices and license terms.
+# This file is part of Tryton.  The COPYRIGHT file at the top level of
+# this repository contains the full copyright notices and license terms.
 "Translate"
 import os
 import locale
 import gettext
-from version import PACKAGE
-from tryton.client import CURRENT_DIR, PREFIX
+from tryton.config import CURRENT_DIR
 import logging
 import gtk
+import sys
 
 _ = gettext.gettext
 
@@ -149,14 +149,12 @@ _LOCALE2WIN32 = {
     'zh_TW': 'Chinese_Taiwan',
 }
 
-_DATE_FORMAT = '%m/%d/%Y'
-
 
 def setlang(lang=None, locale_dict=None):
     "Set language"
     locale_dir = os.path.join(CURRENT_DIR, 'share/locale')
     if not os.path.isdir(locale_dir):
-        locale_dir = os.path.join(PREFIX, 'share/locale')
+        locale_dir = os.path.join(sys.prefix, 'share/locale')
     if lang:
         encoding = locale.getdefaultlocale()[1]
         if not encoding:
@@ -169,8 +167,7 @@ def setlang(lang=None, locale_dict=None):
             lang2 = lang
             if os.name == 'nt':
                 lang2 = _LOCALE2WIN32.get(lang, lang)
-            elif os.name == 'mac' or \
-                    (hasattr(os, 'uname') and os.uname()[0] == 'Darwin'):
+            elif sys.platform == 'darwin':
                 encoding = 'UTF-8'
             # ensure environment variable are str
             lang, lang2, encoding = str(lang), str(lang2), str(encoding)
@@ -179,33 +176,25 @@ def setlang(lang=None, locale_dict=None):
             os.environ['LC_MESSAGES'] = lang2 + '.' + encoding
             os.environ['LANG'] = lang + '.' + encoding
             locale.setlocale(locale.LC_ALL, lang2 + '.' + encoding)
-        except Exception:
-            logging.getLogger('translate').info(
+        except locale.Error:
+            logging.getLogger(__name__).info(
                     _('Unable to set locale %s') % lang2 + '.' + encoding)
 
     if os.path.isdir(locale_dir):
-        gettext.bindtextdomain(PACKAGE, locale_dir)
-    gettext.textdomain(PACKAGE)
+        gettext.bindtextdomain('tryton', locale_dir)
+    gettext.textdomain('tryton')
 
     if locale_dict:
         conv = locale.localeconv()
         for field in locale_dict.keys():
             if field == 'date':
-                global _DATE_FORMAT
-                _DATE_FORMAT = str(locale_dict[field]).replace('%-', '%')
                 continue
             conv[field] = locale_dict[field]
         locale.localeconv = lambda: conv
 
+
 def set_language_direction(direction):
     if direction == 'rtl':
         gtk.widget_set_default_direction(gtk.TEXT_DIR_RTL)
     else:
         gtk.widget_set_default_direction(gtk.TEXT_DIR_LTR)
-
-def date_format():
-    '''
-    Return the locale date format
-    '''
-    global _DATE_FORMAT
-    return _DATE_FORMAT
